@@ -28,6 +28,7 @@
 \*---------------------------------------------------------------------------*/
 
 #include <cmath>
+#include <climits>
 #include <assert.h>
 #include <errno.h>
 #include <pthread.h>
@@ -59,7 +60,7 @@ typedef struct {
 
 struct cref_info {
 	int            cref;
-	struct isdncid cid;
+	VPB_CALL_INFO  cid;
 	int            layer1;    // Encoding as from Call setup or for next call
 	int            transcap;  // Transmit capabilities form Call setup or for next call
 	cref_info     *next;
@@ -115,6 +116,50 @@ static void *audio_thread_log_multi(void *pv);
 #define CH_NEEDONHK	12
 
 
+static void set_callinfo(struct pri_chan &chan, const struct cref_info &cref)
+{ //{{{
+	chan.layer1           = cref.layer1;
+	chan.transcap         = cref.transcap;
+	chan.cid->callingplan = cref.cid.callingplan;
+	chan.cid->callingpres = cref.cid.callingpres;
+	chan.cid->calledplan  = cref.cid.calledplan;
+	strcpy(chan.cid->callingnum,     cref.cid.callingnum);
+	strcpy(chan.cid->callingname,    cref.cid.callingname);
+	strcpy(chan.cid->callingsubaddr, cref.cid.callingsubaddr);
+	strcpy(chan.cid->calledsubaddr,  cref.cid.calledsubaddr);
+	strcpy(chan.cid->callednum,      cref.cid.callednum);
+} //}}}
+
+static void set_callinfo(struct pri_chan &chan, const pri_event_ring &ring_evt)
+{ //{{{
+	chan.cref             = ring_evt.cref;
+	chan.layer1           = ring_evt.layer1;
+	chan.transcap         = ring_evt.ctype;
+	chan.cid->callingplan = ring_evt.callingplan;
+	chan.cid->callingpres = ring_evt.callingpres;
+	chan.cid->calledplan  = ring_evt.calledplan;
+	strcpy(chan.cid->callingnum,     ring_evt.callingnum);
+	strcpy(chan.cid->callingname,    ring_evt.callingname);
+	strcpy(chan.cid->callingsubaddr, ring_evt.callingsubaddr);
+	strcpy(chan.cid->calledsubaddr,  ring_evt.calledsubaddr);
+	strcpy(chan.cid->callednum,      ring_evt.callednum);
+} //}}}
+
+static void set_callinfo(struct cref_info &cref, const pri_event_ring &ring_evt)
+{ //{{{
+	cref.cref            = ring_evt.cref;
+	cref.layer1          = ring_evt.layer1;
+	cref.transcap        = ring_evt.ctype;
+	cref.cid.callingplan = ring_evt.callingplan;
+	cref.cid.callingpres = ring_evt.callingpres;
+	cref.cid.calledplan  = ring_evt.calledplan;
+	strcpy(cref.cid.callingnum,     ring_evt.callingnum);
+	strcpy(cref.cid.callingname,    ring_evt.callingname);
+	strcpy(cref.cid.callingsubaddr, ring_evt.callingsubaddr);
+	strcpy(cref.cid.calledsubaddr,  ring_evt.calledsubaddr);
+	strcpy(cref.cid.callednum,      ring_evt.callednum);
+} //}}}
+
 void OpenPri::HostDSPOpen()
 { //{{{
 	mprintf("[%d/-] Initialising PRI Host DSP ...\n", m_reg.cardnum);
@@ -152,14 +197,13 @@ void OpenPri::HostDSPOpen()
 		chans[i].chan_num = isdn_num;	// Channel number (not true ISDN as doesnt map above d-chan)
 		chans[i].cref = 0;
 		chans[i].vox_states = NULL;
-#ifdef SOFTBRIDGE
+
+            #if defined(CASLOG) || defined(SOFTBRIDGE)
 		hostvox_open(&chans[i].vox_states,i);
-#endif
-#ifdef CASLOG
-		if(islog && (chans[i].vox_states == NULL)){
-			hostvox_open(&chans[i].vox_states,i);
-		}
-#endif
+            #endif
+            #ifdef CASLOG
+		chans[i].run = 1;
+            #endif
 		if (iface_num >= dchan){
 			chans[i].iface = &cardinfo->iface[iface_num+1];
 		}
@@ -229,17 +273,18 @@ void OpenPri::HostDSPOpen()
 		       &m_audio_multi_info[x]);
 	mprintf("[%d/-] Started Audio Thread Multi [%d]...\n",m_reg.cardnum,x);
 
-
 	int ret = pthread_create(&m_dspthread, NULL, SimulateDSP, this);
 	if( ret )
 		mprintf("[%d/-] FAILED to start DSP thread: %s\n",
 			m_reg.cardnum, strerror(ret));
 
+    #ifndef CASLOG
 	ret = pthread_create(&m_dchanthread, NULL,
 			     islog ? RunDchannelLog : RunDchannel, this);
 	if( ret )
 		mprintf("[%d/-] FAILED to start D channel thread: %s\n",
 			m_reg.cardnum, strerror(ret));
+    #endif
 } //}}}
 
 void OpenPri::HostDSPClose()
@@ -248,10 +293,15 @@ void OpenPri::HostDSPClose()
 	int cardnum = m_reg.cardnum;
 
 	mprintf("[%d/-] Closing Pri Host DSP...\n",cardnum);
+    #ifdef CASLOG
+	pthread_cancel(m_dspthread);
+	pthread_join(m_dspthread, NULL);
+    #else
 	pthread_cancel(m_dspthread);
 	pthread_cancel(m_dchanthread);
 	pthread_join(m_dspthread, NULL);
 	pthread_join(m_dchanthread, NULL);
+    #endif
 
 	for(int i = 0; i < m_audio_multi; ++i) {
 		pthread_cancel(m_audiothread[i]);
@@ -459,7 +509,25 @@ void prepare_rec_buf(unsigned char alawbuf[NBUF], VPBREG *v, int ch, int encodin
 #else
 	v->rxdf[ch]->Write( (uint16_t*)linbuf, NBUF);
 #endif
-
+#ifdef CASLOG
+	if (chans[idx].vox_states){
+		ret = hostvox_analyse(chans[idx].vox_states,linbuf,NBUF);
+		if (ret == 1){ // Vox OFF
+			mess[0] = DSP_LVOXOFF;
+			mess[1] = DSP_VOXOFF;
+			mess[2] = chans[idx].chan_num;
+			ret = v->upmess->Write(mess, mess[0]);
+			mprintf("ch = %d bd = %d VOX OFF\n", ch, v->cardnum);
+		}
+		else if (ret ==2){ // Vox ON
+			mess[0] = DSP_LVOXON;
+			mess[1] = DSP_VOXON;
+			mess[2] = chans[idx].chan_num;
+			ret = v->upmess->Write(mess, mess[0]);
+			mprintf("ch = %d bd = %d VOX ON\n", ch, v->cardnum);
+		}
+	}
+#endif
 	// DTMF processing
 	digits_ret = dtmf_decoder(chans[idx].dtmf_dec, key, 2, linbuf, NBUF, &keydown);
 	if (digits_ret) {
@@ -646,7 +714,7 @@ bool OpenPri::ProcessCommands()
 	OP_CARD  *cardinfo = (OP_CARD *)m_reg.cardinfo;
 	int       cardnum  = m_reg.cardnum;
 	int	  ret;
-	isdncid	 *tmpp;
+	VPB_CALL_INFO	*tmpp;
 
 	if( m_reg.dnmess->Read(&mess[0], 1) == Fifo::EMPTY )
 		return true;  //DSP idle
@@ -673,18 +741,23 @@ bool OpenPri::ProcessCommands()
 	    }
 	}
 	//mprintf("message bytes[%d]: on ch[%d]\n",mess[0],mess[2]);
-	if( cardinfo->node == ISDN_NODE_LOG ) {
-	//	mprintf("[%d/-] Ignoring command, in logging mode \n",cardnum);
-		if( mess[1] == PC_VOX_UPDATE ) {
-			assert(mess[0] == PC_LVOX_UPDATE);
-			if(chans[idx].vox_states != NULL){
-				hostvox_update_levels(chans[idx].vox_states, mess);
-				//mprintf("[%d/-]Updating vox information\n",cardnum);
-			} else  mprintf("[%d/-]No vox_states !\n",cardnum);
-		}
-		return false;
+
+	if( mess[1] == PC_VOX_UPDATE ) {
+		assert(mess[0] == PC_LVOX_UPDATE);
+		if(chans[idx].vox_states != NULL){
+			hostvox_update_levels(chans[idx].vox_states, mess);
+			//mprintf("[%d/-]Updating vox information\n",cardnum);
+		} else  mprintf("[%d/-]No vox_states !\n",cardnum);
 	}
 
+	// If this is CAS, or we are in logging mode,
+	// then don't process any other 'dsp' messages.
+    #ifndef CASLOG
+	if( cardinfo->node == ISDN_NODE_LOG )
+    #endif
+		return false;
+
+
 	switch(mess[1]) {
 	    case PC_CODEC_ONHK:
 		assert(mess[0] == PC_LCODEC_ONHK);
@@ -886,17 +959,6 @@ bool OpenPri::ProcessCommands()
 		toned_start_logging(m_reg.toned[ch], mess);
 		break;
 
-	    case PC_VOX_UPDATE:
-		assert(mess[0] == PC_LVOX_UPDATE);
-		if(chans[idx].vox_states != NULL){
-			hostvox_update_levels(chans[idx].vox_states, mess);
-			mprintf("[%d/-]Updating vox information\n",cardnum);
-		}
-		else {
-			mprintf("[%d/-]No vox_states !\n",cardnum);
-		}
-		break;
-
 	    case PC_ISDN_CALL:
 		mprintf("[%d/%d] Making an ISDN call!\n",
 			cardnum,chans[idx].chan_num);
@@ -922,7 +984,7 @@ bool OpenPri::ProcessCommands()
 			idx,cardnum,chans[idx].chan_num,mess[3],mess[4],mess[5]);
 		memcpy(&tmpp,&mess[4],mess[3]);
 		if ((tmpp != NULL)&&(chans[idx].cid != NULL)){
-			memcpy(tmpp,chans[idx].cid,sizeof(isdncid));
+			memcpy(tmpp,chans[idx].cid,sizeof(VPB_CALL_INFO));
 		}
 //			mprintf("[%d/-]Getting Calling Number: cnum(%d)\n",cardnum,tchp);
 //			strcpy(tchp,chans[idx].cid->callingnum);
@@ -969,11 +1031,6 @@ static void RunDchannel_cleanup( void *p )
 void *OpenPri::RunDchannel(void *p)
 { //{{{
 	OpenPri* openpri = (OpenPri*)p;
-	int dfd = 0;
-	int swtype = PRI_SWITCH_EUROISDN_E1;
-	int node = PRI_CPE;
-	//struct wan_sockaddr_ll  sa;
-	int d_chan=-1;
 
 	pri_event *e;
 	struct timeval tv = {0,0}, *next, ts;
@@ -988,25 +1045,17 @@ void *OpenPri::RunDchannel(void *p)
 	int numch      = v->numch;
 
 	OP_CARD *cardinfo = (OP_CARD *)v->cardinfo;
-
-	pri_chan *chans;
-	chans = (pri_chan *)v->chans;
-	// Initialise channel states
-	for(; x < numch; ++x) {
-		chans[x].state = CH_IDLE;
-		chans[x].run   = 0;
-	}
+	pri_chan *chans   = (pri_chan *)v->chans;
 
 	// Set switch type from config file
-	swtype = cardinfo->protocol;
-	node = cardinfo->node;
-	if (node == ISDN_NODE_LOG) return RunDchannelLog(p);
+	int swtype = cardinfo->protocol;
+	int node = cardinfo->node;
 
 	// Get D-channel from config file
-	d_chan = cardinfo->d_channel;
+	int d_chan = cardinfo->d_channel;
 
-	dfd = wanconnect((char*)cardinfo->iface[d_chan].card,
-			 (char*)cardinfo->iface[d_chan].name);
+	int dfd = wanconnect((char*)cardinfo->iface[d_chan].card,
+			     (char*)cardinfo->iface[d_chan].name);
 	if (dfd < 0){
 		mprintf("[%d/-] Failed[%d] to open dchannel '%s'\n",
 			cardnum, dfd, cardinfo->iface[d_chan].name);
@@ -1157,12 +1206,13 @@ void *OpenPri::RunDchannel(void *p)
 
 			    case CH_NEEDRING:
 				if (chans[x].cid)
-					mprintf("[%d/%02d] Sending ring event up...(cref:%d)(%s %s %s %s)\n",
+					mprintf("[%d/%02d] Sending ring event up...(cref:%d)(%s %s %s %s %s)\n",
 						cardnum,
 						chans[x].chan_num,
 						chans[x].cref,
 						chans[x].cid->callingnum,
 						chans[x].cid->callingname,
+						chans[x].cid->callingsubaddr,
 						chans[x].cid->callednum,
 						chans[x].cid->calledsubaddr);
 				else
@@ -1189,22 +1239,14 @@ void *OpenPri::RunDchannel(void *p)
 void *OpenPri::RunDchannelLog(void *p)
 { //{{{
 	OpenPri* openpri = (OpenPri*)p;
-	int swtype = PRI_SWITCH_EUROISDN_E1;
-	int node = PRI_CPE;
-	//struct wan_sockaddr_ll  sa;
-	int d_chan=-1;
 
 	pri_event *e;
 	struct timeval ts;
-	int dfd;
-	int dfd2;
-	int seldfd;
 	fd_set rfds, efds;
 	int res, i, x = 0;
 	struct pri *pri;
 	struct pri *pri2;
 	struct pri *pritmp = NULL;
-	int islog=0;
 
 	char *tmpstr;
 
@@ -1218,34 +1260,11 @@ void *OpenPri::RunDchannelLog(void *p)
 	int numch      = v->numch;
 
 	OP_CARD *cardinfo = (OP_CARD *)v->cardinfo;
-
-	pri_chan *chans;
-	chans = (pri_chan *)v->chans;
-
-	// Initialise channel states
-	for(; x < numch; ++x) {
-		chans[x].state = CH_IDLE;
-	    #ifdef CASLOG
-		chans[x].run = 1;
-	    #else
-		chans[x].run = 0;
-	    #endif
-	}
+	pri_chan *chans   = (pri_chan *)v->chans;
 
 	// Set switch type from config file
-	swtype = cardinfo->protocol;
-	node = cardinfo->node;
-	if (node == ISDN_NODE_LOG) {
-		node = PRI_CPE;
-		islog=1;
-	} else {
-		return RunDchannel(p);
-	}
-
-      #ifdef CASLOG
-	mprintf("[%d/-] D-Channel is disabled in cas logging mode!\n",cardnum);
-	pthread_exit(NULL);
-      #endif
+	int swtype = cardinfo->protocol;
+	int node   = PRI_CPE;
 
 	// Split up channel and card names
 	// iface is initialised from config file which uses channel1 - channel31
@@ -1267,12 +1286,12 @@ void *OpenPri::RunDchannelLog(void *p)
 		}
 	}
 	// Get D-channel from config file
-	d_chan = cardinfo->d_channel;
+	int d_chan = cardinfo->d_channel;
 
-	dfd = wanconnect((char*)cardinfo->iface[d_chan].card,
-			 (char*)cardinfo->iface[d_chan].name);
-	dfd2 = wanconnect((char*)cardinfo->iface[d_chan].card2,
-			  (char*)cardinfo->iface[d_chan].name2);
+	int dfd = wanconnect((char*)cardinfo->iface[d_chan].card,
+			     (char*)cardinfo->iface[d_chan].name);
+	int dfd2 = wanconnect((char*)cardinfo->iface[d_chan].card2,
+			      (char*)cardinfo->iface[d_chan].name2);
 	if( dfd < 0 || dfd2 < 0 ){
 		mprintf("[%d/-] Failed[%d] to open dchannel '%s'\n",
 			cardnum, dfd, cardinfo->iface[d_chan].name);
@@ -1284,7 +1303,8 @@ void *OpenPri::RunDchannelLog(void *p)
 			cardnum,dfd2,cardinfo->iface[d_chan].name2);
 	}
 	mprintf("[%d/-] D-Channel is in logging mode!\n",cardnum);
-	seldfd = dfd2 > dfd ? dfd2 : dfd;
+
+	int seldfd = dfd2 > dfd ? dfd2 : dfd;
 
 	DchannelData	cleanup( cardnum, dfd, dfd2 );
 	pthread_cleanup_push(RunDchannel_cleanup, &cleanup);
@@ -1403,17 +1423,10 @@ void *OpenPri::RunDchannelLog(void *p)
 								mprintf("[%d/%02d] Found call information from cref_info linked list\n",
 									cardnum,chans[x].chan_num);
 								if (chans[x].cid == NULL){
-									chans[x].cid = new isdncid;
+									chans[x].cid = new VPB_CALL_INFO;
 								}
-								chans[x].cid->callingplan = cref_info->cid.callingplan;
-								chans[x].cid->callingpres = cref_info->cid.callingpres;
-								chans[x].cid->calledplan  = cref_info->cid.calledplan;
-								strcpy(chans[x].cid->callingnum,  cref_info->cid.callingnum);
-								strcpy(chans[x].cid->callingname, cref_info->cid.callingname);
-								strcpy(chans[x].cid->callednum,   cref_info->cid.callednum);
+								set_callinfo(chans[x], *cref_info);
 								chans[x].cref_check = 1;
-								chans[x].layer1     = cref_info->layer1;
-								chans[x].transcap   = cref_info->transcap;
 								if(cref_info_last != NULL){
 									cref_info_last->next = cref_info->next;
 								}
@@ -1499,9 +1512,9 @@ void OpenPri::handle_pri_event(struct pri *pri, pri_event *e)
 		break;
 
 	    case PRI_EVENT_RING:
-		mprintf("[%d/%d] PRI Ring on channel (from [%s] to [%s])(cref:%d)\n",
+		mprintf("[%d/%d] PRI Ring on channel (from [%s] to [%s] sub [%s])(cref:%d)\n",
 			m_reg.cardnum, e->ring.channel, e->ring.callingnum,
-			e->ring.callednum,e->ring.cref);
+			e->ring.callednum,e->ring.calledsubaddr,e->ring.cref);
 		if (e->ring.channel != -1){
 			try {
 				idx = get_idx_from_ich(&m_reg,e->ring.channel);
@@ -1525,18 +1538,9 @@ void OpenPri::handle_pri_event(struct pri *pri, pri_event *e)
 		chans[idx].call = e->ring.call;
 		/* Save caller ID info */
 		if (chans[idx].cid == NULL){
-			chans[idx].cid = new isdncid;
+			chans[idx].cid = new VPB_CALL_INFO;
 		}
-		chans[idx].cid->callingplan = e->ring.callingplan;
-		chans[idx].cid->callingpres = e->ring.callingpres;
-		chans[idx].cid->calledplan = e->ring.calledplan;
-		strcpy(chans[idx].cid->callingnum,e->ring.callingnum);
-		strcpy(chans[idx].cid->callingname,e->ring.callingname);
-		strcpy(chans[idx].cid->calledsubaddr,e->ring.calledsubaddr);
-		strcpy(chans[idx].cid->callednum,e->ring.callednum);
-		chans[idx].cref=e->ring.cref;
-		chans[idx].layer1=e->ring.layer1;
-		chans[idx].transcap=e->ring.ctype;
+		set_callinfo(chans[idx], e->ring);
 		for(i=0;i<e->ring.lowlayercompatibility[0]+1;i++){
 			chans[idx].lowlayercompatibility[i]=e->ring.lowlayercompatibility[i];
 		}
@@ -1678,9 +1682,9 @@ void OpenPri::handle_pri_event_log(struct pri *pri, pri_event *e)
 		break;
 
 	    case PRI_EVENT_RING:
-		mprintf("[%d/%d] PRI Ring on channel (from [%s] to [%s])(cref:%d)\n",
+		mprintf("[%d/%d] PRI Ring on channel (from [%s] to [%s] sub [%s])(cref:%d)\n",
 			m_reg.cardnum, e->ring.channel, e->ring.callingnum,
-			e->ring.callednum,e->ring.cref);
+			e->ring.callednum,e->ring.calledsubaddr,e->ring.cref);
 		if (e->ring.channel != -1) {
 			try {
 				if (idx == -1)
@@ -1696,18 +1700,10 @@ void OpenPri::handle_pri_event_log(struct pri *pri, pri_event *e)
 			chans[idx].call = e->ring.call;
 			/* Save caller ID info */
 			if (chans[idx].cid == NULL){
-				chans[idx].cid = new isdncid;
+				chans[idx].cid = new VPB_CALL_INFO;
 			}
-			chans[idx].cid->callingplan = e->ring.callingplan;
-			chans[idx].cid->callingpres = e->ring.callingpres;
-			chans[idx].cid->calledplan = e->ring.calledplan;
-			strcpy(chans[idx].cid->callingnum,e->ring.callingnum);
-			strcpy(chans[idx].cid->callingname,e->ring.callingname);
-			strcpy(chans[idx].cid->callednum,e->ring.callednum);
-			chans[idx].cref=e->ring.cref;
+			set_callinfo(chans[idx], e->ring);
 			chans[idx].cref_check=1;
-			chans[idx].layer1=e->ring.layer1;
-			chans[idx].transcap=e->ring.ctype;
 			pthread_mutex_unlock(&m_mutex);
 		}
 		else {
@@ -1723,15 +1719,7 @@ void OpenPri::handle_pri_event_log(struct pri *pri, pri_event *e)
 				m_cref_info = crefinfo;
 			}
 			crefinfo->next = NULL;
-			crefinfo->cid.callingplan = e->ring.callingplan;
-			crefinfo->cid.callingpres = e->ring.callingpres;
-			crefinfo->cid.calledplan  = e->ring.calledplan;
-			strcpy(crefinfo->cid.callingnum,  e->ring.callingnum);
-			strcpy(crefinfo->cid.callingname, e->ring.callingname);
-			strcpy(crefinfo->cid.callednum,   e->ring.callednum);
-			crefinfo->cref     = e->ring.cref;
-			crefinfo->layer1   = e->ring.layer1;
-			crefinfo->transcap = e->ring.ctype;
+			set_callinfo(*crefinfo, e->ring);
 		}
 		break;
 
@@ -1965,18 +1953,10 @@ void OpenPri::handle_pri_event_log(struct pri *pri, pri_event *e)
 			chans[idx].call = e->ring.call;
 			/* Save caller ID info */
 			if (chans[idx].cid == NULL){
-				chans[idx].cid = new isdncid;
+				chans[idx].cid = new VPB_CALL_INFO;
 
-				chans[idx].cid->callingplan = e->ring.callingplan;
-				chans[idx].cid->callingpres = e->ring.callingpres;
-				chans[idx].cid->calledplan = e->ring.calledplan;
-				strcpy(chans[idx].cid->callingnum,e->ring.callingnum);
-				strcpy(chans[idx].cid->callingname,e->ring.callingname);
-				strcpy(chans[idx].cid->callednum,e->ring.callednum);
-				chans[idx].cref=e->ring.cref;
+				set_callinfo(chans[idx], e->ring);
 				chans[idx].cref_check=1;
-				chans[idx].layer1=e->ring.layer1;
-				chans[idx].transcap=e->ring.ctype;
 			} else {
 
 				// Collect the pri numbers coming thru in INFO events
@@ -2628,6 +2608,12 @@ void OpenPri::WaitForTxEmpty( int port )
 #endif
 } //}}}
 
+void OpenPri::UNListen( unsigned int port )
+{ //{{{
+	mprintf("[%d/%d] OpenPri::UNListen: not implemented\n",
+		m_reg.cardnum, port);
+} //}}}
+
 /*--------------------------------------------------------------------------*\
 
 	FUNCTION.: OpenPri::chan_state()
@@ -2698,13 +2684,13 @@ int OpenPri::chan_transcap(int ch)
 	Provides the call info of a channel
 
 \*--------------------------------------------------------------------------*/
-int OpenPri::chan_cinfo(int ch, void *info)
+int OpenPri::chan_cinfo(int ch, VPB_CALL_INFO *info)
 {
 	try {
 		int idx = get_idx_from_ch(&m_reg,ch);
 		pri_chan *chans = (pri_chan *)m_reg.chans;
 		if( info != NULL && chans[idx].cid != NULL ){
-			memcpy(info,chans[idx].cid,sizeof(isdncid));
+			memcpy(info,chans[idx].cid,sizeof(VPB_CALL_INFO));
 			return 0;
 		}
 	} catch ( const Wobbly &w ) {
