@@ -242,7 +242,7 @@ enum VPB_PORT
     //!       If you need to check the model programmatically, use the
     //!       @c vpb_get_card_type() function instead.
     //}}}
-    std::string WINAPI vpb_get_model(int handle);
+    std::string WINAPI vpb_get_model(VPBPortHandle handle);
 
     //! Return a string indicating the board model.
     //{{{
@@ -255,13 +255,13 @@ enum VPB_PORT
     //!       If you need to check the model programmatically, use the
     //!       @c vpb_get_card_type() function instead.
     //}}}
-    int WINAPI vpb_get_model(int handle, char *str);
+    int WINAPI vpb_get_model(VPBPortHandle handle, char *str);
 
     //! Return a descriptive string for a @c VPB_MODEL type.
     const char *WINAPI vpb_model_desc(VPB_MODEL model);
 
     //! Return the board model type for @a handle.
-    VPB_MODEL WINAPI vpb_get_card_type(int handle);
+    VPB_MODEL WINAPI vpb_get_card_type(VPBPortHandle handle);
 
     //! Return the @c VPB_PORT type for @a handle.
     VPB_PORT WINAPI vpb_get_port_type(VPBPortHandle handle);
@@ -514,14 +514,21 @@ VT_DEPRECATED( int WINAPI vpb_conf(int h1,int resource, int mode) );
 
   //@} //}}}
 
-    int WINAPI vpb_enable_event(int handle, unsigned long mask);
-    int WINAPI vpb_disable_event(int handle, unsigned long mask);
+    //! Enable the events specified by @a mask for @a handle.
+    //
+    //! Events not included in the @a mask are unaffected.
+    int WINAPI vpb_enable_event(VPBPortHandle handle, unsigned long mask);
+
+    //! Disable the events specified by @a mask for @a handle.
+    //
+    //! Events not included in the @a mask are unaffected.
+    int WINAPI vpb_disable_event(VPBPortHandle handle, unsigned long mask);
 
     //! Return the current event mask for @a handle.
-    unsigned long WINAPI vpb_get_event_mask(int handle);
+    unsigned long WINAPI vpb_get_event_mask(VPBPortHandle handle);
 
     //! Set the event mask for @a handle.
-    int WINAPI vpb_set_event_mask(int handle, unsigned long mask);
+    int WINAPI vpb_set_event_mask(VPBPortHandle handle, unsigned long mask);
 
     //! Fetch the next event from the global event queue.
     //{{{
@@ -558,7 +565,7 @@ VT_DEPRECATED( int WINAPI vpb_conf(int h1,int resource, int mode) );
     //!
     //! @sa vpb_get_event_async(), vpb_get_event_sync(), vpb_get_event_ch_sync()
     //}}}
-    int WINAPI vpb_get_event_ch_async(int handle, VPB_EVENT *event);
+    int WINAPI vpb_get_event_ch_async(VPBPortHandle handle, VPB_EVENT *event);
 
     //! Wait for an event on the global event queue.
     //{{{
@@ -601,7 +608,7 @@ VT_DEPRECATED( int WINAPI vpb_conf(int h1,int resource, int mode) );
     //!
     //! @sa vpb_get_event_sync(), vpb_get_event_async(), vpb_get_event_ch_async()
     //}}}
-    int WINAPI vpb_get_event_ch_sync(int handle, VPB_EVENT *event,
+    int WINAPI vpb_get_event_ch_sync(VPBPortHandle handle, VPB_EVENT *event,
 				     unsigned int time_out = 0);
 
     //! Places an @a event on the global and port specific event queues.
@@ -650,7 +657,7 @@ VT_DEPRECATED( int WINAPI vpb_conf(int h1,int resource, int mode) );
     //! @todo Should this block until it is safe to release the @a context data
     //!       and/or the function definition itself?
     //}}}
-    int WINAPI vpb_set_event_callback( int handle,
+    int WINAPI vpb_set_event_callback( VPBPortHandle handle,
 				       vpb_event_callback callback,
 				       void *context = NULL);
 
@@ -658,7 +665,13 @@ VT_DEPRECATED( int WINAPI vpb_conf(int h1,int resource, int mode) );
     int WINAPI vpb_get_event_count(void);
 
     //! Returns the number of events on the port specific event queue for @a handle.
-    int WINAPI vpb_get_event_count_ch(int handle);
+    int WINAPI vpb_get_event_count_ch(VPBPortHandle handle);
+
+    //! Clear all unread events from the queue for @a handle.
+    void WINAPI vpb_flush_events(VPBPortHandle handle);
+
+    //! Clear all unread events from the global event queue.
+    void WINAPI vpb_flush_events();
 
 //!@} //}}}
 
@@ -1645,16 +1658,24 @@ VT_DEPRECATED( int WINAPI vpb_conf(int h1,int resource, int mode) );
     } //}}}
 
 
-    int WINAPI vpb_debug_tonedet(int handle, int id, char file_name[], int sec);
+    int WINAPI vpb_debug_tonedet(VPBPortHandle handle, int id, char file_name[], int sec);
     //int WINAPI vpb_tonedet_make_default(VPB_DETECT *d);
 
-    int WINAPI vpb_enable_tone_event(int handle, unsigned long mask);
-    int WINAPI vpb_disable_tone_event(int handle, unsigned long mask);
+    //! Enable the @c TONE events specified by @a mask for @a handle.
+    //
+    //! Events not included in the @a mask are unaffected.
+    int WINAPI vpb_enable_tone_event(VPBPortHandle handle, unsigned long mask);
+
+    //! Disable the @c TONE events specified by @a mask for @a handle.
+    //
+    //! Events not included in the @a mask are unaffected.
+    int WINAPI vpb_disable_tone_event(VPBPortHandle handle, unsigned long mask);
 
     //! Return the current @c TONE event mask for @a handle.
-    unsigned long WINAPI vpb_get_tone_event_mask(int handle);
+    unsigned long WINAPI vpb_get_tone_event_mask(VPBPortHandle handle);
 
-    int WINAPI vpb_set_tone_event_mask(int handle, unsigned long mask);
+    //! Set the @c TONE event mask.  The previous mask value will be overwritten.
+    int WINAPI vpb_set_tone_event_mask(VPBPortHandle handle, unsigned long mask);
 
 //!@} //}}}
 
@@ -1722,10 +1743,10 @@ VT_DEPRECATED( int WINAPI vpb_conf(int h1,int resource, int mode) );
     //
     //! This will override the default country data for @a handle. If @a country
     //! is @c NULL, then @a handle will revert to using the global default data.
-    int WINAPI vpb_set_country(int handle, const Country *country);
+    int WINAPI vpb_set_country(VPBPortHandle handle, const Country *country);
 
     //! Get the current country data for port @a handle.
-    const Country* WINAPI vpb_get_port_country(int handle);
+    const Country* WINAPI vpb_get_port_country(VPBPortHandle handle);
 
     //! Get country specific data by name.
     //
@@ -1927,7 +1948,7 @@ int  WINAPI vpb_adpcm_decode(void  *adpcm,
     { //{{{
 
 	//! A list of specific digits that may terminate collection.
-	char	      *term_digits;
+	const char    *term_digits;
 
 	//! The maximum number of digits to collect.
 	unsigned short max_digits;
@@ -1969,7 +1990,7 @@ int  WINAPI vpb_adpcm_decode(void  *adpcm,
 
     //! Get a buffer of @c DTMF digits.
     //{{{
-    //! This functions will return immediately and post a @c VPB_DIGIT event
+    //! This function will return immediately and post a @c VPB_DIGIT event
     //! for @a handle when collection is completed.  If it is called while
     //! collection is already in progress for @a handle, the process is reset
     //! and any digits collected prior to that will be discarded.
@@ -1988,10 +2009,10 @@ int  WINAPI vpb_adpcm_decode(void  *adpcm,
 
     //! Get a buffer of @c DTMF digits.
     //{{{
-    //! This functions will block until collection is completed.  If it is
+    //! This function will block until collection is completed.  If it is
     //! called while async collection is already in progress for @a handle,
     //! the process is reset and any digits collected prior to that will be
-    //! discarded.  If it is called while synchronous collections is already
+    //! discarded.  If it is called while synchronous collection is already
     //! in progress an exception will be thrown.  If it is interrupted by a
     //! call to start async collection an exception will be thrown.
     //!
@@ -2004,6 +2025,16 @@ int  WINAPI vpb_adpcm_decode(void  *adpcm,
     //! @return The @c CollectEndReason indicating why collection was terminated.
     //!
     //! @exception various exceptions may be thrown in the event of an error.
+    //!
+    //! @note In most cases, this probably isn't the function you want and you
+    //!       should use @c vpb_get_digits_async() instead.  This function does
+    //!       <b>no checking</b> for other events (such as hangup) while it is
+    //!       waiting for digits and there is no graceful way to interrupt it
+    //!       if you are checking for them elsewhere in some other thread.
+    //!       Most code should integrate the async digit collection with its
+    //!       normal event processing loop, as the cases where DTMF digits
+    //!       really are the @a only thing you need to be watching for tend to
+    //!       be the exception rather than the rule.
     //}}}
     CollectEndReason WINAPI vpb_get_digits_sync(int handle, VPB_DIGITS *digits, char *digbuf);
 
@@ -2354,6 +2385,7 @@ int  WINAPI vpb_adpcm_decode(void  *adpcm,
 	    int callingpres;
 	    char callingnum[256];
 	    char callingname[256];
+	    char callingsubaddr[256];
 	    char calledsubaddr[256];
 	    char callednum[256];
 	    int calledplan;
