@@ -91,13 +91,7 @@ static int waitforevent(int h, int event_type, int timeout_ms)
 	return event;
 } //}}}
 
-static void port_thread_cleanup( void *p )
-{ //{{{
-	int         h    = (long)p;
-	VPB_PORT    type = vpb_get_port_type(h);
-
-	if(type == VPB_FXO) vpb_sethook_sync(h, VPB_ONHOOK);
-} //}}}
+static void port_thread_cleanup( void *p ) { vpb_reset( (intptr_t)p ); }
 
 static void *port_thread(void *p)
 { //{{{
@@ -127,7 +121,7 @@ static void *port_thread(void *p)
 		if( ev == VPB_RING) ev = waitforevent(h, VPB_RING, 6000);
 		if( ev == VPB_RING || ev == VPB_STATION_OFFHOOK )
 		{
-			// Ensure we go back on hook again if terminated.
+			// Ensure we clean up the port state if terminated.
 			pthread_cleanup_push(port_thread_cleanup, p);
 
 			if( type == VPB_FXO ) {
@@ -142,9 +136,8 @@ static void *port_thread(void *p)
 			printf("[%02d] playing.....\n",h);
 			vpb_play_file_sync(h, file_name);
 
-			pthread_cleanup_pop(1);
-			//if(type == VPB_FXO) vpb_sethook_sync(h, VPB_ONHOOK);
 			printf("[%02d] done.....\n",h);
+			pthread_cleanup_pop(1);
 		}
 	}
 
@@ -203,8 +196,7 @@ int main(int argc, char *argv[])
 			int h = vpb_open(i,j);
 
 			vpb_record_set(h, record);
-			pthread_create(&port_threads[x], NULL, port_thread, (void*)h);
-			++x;
+			pthread_create(&port_threads[x++], NULL, port_thread, (void*)h);
 
 			//int y = arg_exists(argc, argv, "--hookthreshold");
 			//if( y != 0 && (y+1) < argc)
