@@ -2168,8 +2168,8 @@ static int create_module_proc_const_int(struct openpci *card, int module,
 
 	p = create_proc_read_entry(node, 0444, card->moduleinfo[module].procfs_root,
 					 vt_proc_read_int, (void*)val);
-	if( p ){
-		p->owner = THIS_MODULE;
+	if (p) {
+		set_proc_owner(p, THIS_MODULE);
 		return RET_OK;
 	}
 	crit("failed to create procfs module%d/%s", module, node);
@@ -2184,7 +2184,7 @@ static int create_module_proc_const_string(struct openpci *card, int module,
 	p = create_proc_read_entry(node, 0444, card->moduleinfo[module].procfs_root,
 					 vt_proc_read_string, (void*)str);
 	if( p ){
-		p->owner = THIS_MODULE;
+		set_proc_owner(p, THIS_MODULE);
 		return RET_OK;
 	}
 	crit("failed to create procfs module%d/%s", module, node);
@@ -2212,7 +2212,7 @@ static void export_module_info( struct openpci *card )
 			cardcrit(card->cardnum, "failed to create procfs module%d", i);
 			continue;
 		}
-		(*modroot)->owner = THIS_MODULE;
+		set_proc_owner(*modroot, THIS_MODULE);
 
 		create_module_proc_const_string(card, i, "type", mod->type );
 		create_module_proc_const_int(card, i, "ports", mod->ports );
